@@ -7,7 +7,10 @@ from numpy import reshape
 from numpy import empty
 from numpy import dstack, vstack
 
-from sklearn.preprocessing import StandardScaler, MinMaxScaler, MaxAbsScaler, FunctionTransformer
+from sklearn.preprocessing import (StandardScaler,
+                                   MinMaxScaler,
+                                   MaxAbsScaler,
+                                   FunctionTransformer)
 
 import pandas as pd
 from pandas import DataFrame
@@ -15,7 +18,16 @@ import os
 
 import tensorflow as tf
 from tensorflow import keras
-from tensorflow.keras.layers import LSTM, Dense, Flatten, Conv1D, MaxPooling1D, Dropout, Bidirectional, TimeDistributed, GRU, SimpleRNN
+from tensorflow.keras.layers import (LSTM,
+                                     Dense,
+                                     Flatten,
+                                     Conv1D,
+                                     MaxPooling1D,
+                                     Dropout,
+                                     Bidirectional,
+                                     TimeDistributed,
+                                     GRU,
+                                     SimpleRNN)
 
 
 class HybridMultStepMultVar(MultiVariateMultiStep):
@@ -49,22 +61,34 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
         get_metrics(self) -> str:
             Getter method to retrieve model evaluation metrics used.
         create_cnnrnn(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Builds CNN-RNN structure.
         create_cnnlstm(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Builds CNN-LSTM structure.
         create_cnngru(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Builds CNN-GRU structure.
         create_cnnbirnn(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Buidls CNN bidirectional RNN structure.
         create_cnnbilstm(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Builds CNN bidirectional LSTM structure.
         create_cnnbigru(self, optimizer: str = 'adam',
-        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error'):
+        loss: str = 'mean_squared_error', metrics: str = 'mean_squared_error',
+        layer_config = {'layer0': (64, 1, 'relu'), 'layer1': (32, 1, 'relu'),
+        'layer2': (2), 'layer3': (50, 'relu'), 'layer4': (25, 'relu')}):
             Builds CNN bidirectional GRU structure.
         fit_model(self, epochs: int, show_progress: int = 1):
             Fitting selected model to data provided.
@@ -282,15 +306,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
         return self.metrics
 
     def create_cnnrnn(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-RNN hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-RNN')
         self.loss = loss
@@ -300,6 +341,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
@@ -307,15 +349,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             output_shape=self.input_y.shape[1])
 
     def create_cnnlstm(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-LSTM hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-LSTM')
         self.loss = loss
@@ -325,6 +384,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
@@ -332,15 +392,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             output_shape=self.input_y.shape[1])
 
     def create_cnngru(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-GRU hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-GRU')
         self.loss = loss
@@ -350,6 +427,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
@@ -357,15 +435,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             output_shape=self.input_y.shape[1])
 
     def create_cnnbirnn(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-BI-RNN hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-BI-RNN')
         self.loss = loss
@@ -375,6 +470,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
@@ -382,15 +478,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             output_shape=self.input_y.shape[1])
 
     def create_cnnbilstm(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-BI-LSTM hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-BI-LSTM')
         self.loss = loss
@@ -400,6 +513,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
@@ -407,15 +521,32 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             output_shape=self.input_y.shape[1])
 
     def create_cnnbigru(
-            self,
-            optimizer: str = 'adam',
-            loss: str = 'mean_squared_error',
-            metrics: str = 'mean_squared_error'):
+        self,
+        optimizer: str = 'adam',
+        loss: str = 'mean_squared_error',
+        metrics: str = 'mean_squared_error',
+        layer_config={
+            'layer0': (
+            64,
+            1,
+            'relu'),
+            'layer1': (
+                32,
+                1,
+                'relu'),
+            'layer2': (2),
+            'layer3': (
+                50,
+                'relu'),
+            'layer4': (
+            25,
+            'relu')}):
         '''Creates CNN-BI-GRU hybrid model.
             Parameters:
                 optimizer (str): Optimization algorithm.
                 loss (str): Loss function.
                 metrics (str): Performance measurement.
+                layer_config (dict): Adjust neurons and acitivation functions.
         '''
         self.set_model_id('CNN-BI-GRU')
         self.loss = loss
@@ -425,6 +556,7 @@ class HybridMultStepMultVar(MultiVariateMultiStep):
             optimizer=optimizer,
             loss=loss,
             metrics=metrics,
+            layer_config=layer_config,
             input_shape=(
                 self.input_x.shape[1],
                 self.input_x.shape[2],
